#include "FGDependencyContext.h"

#include <sstream>
#include <assert.h>

using namespace std;

namespace egstra {
  void fg_dependency_context::phi(const sentence* x, list<string> F[],
				  const int word_limit,
				  const bool use_pos) {
    if(!use_pos) { return; }
    const int N = x->size();
    ostringstream os;
    for(int h = 0; h < N; ++h) {
      for(int m = 0; m < N; ++m) {
	const int dep_idx = h + N*m;
	list<string>& f = F[dep_idx];

	/* the start of the dependency in the sentence */
	const int start = (h < m ? h : m);
	/* the end of the dependency in the sentence */
	const int end = (h < m ? m : h);
	/* indicator for the direction of the dependency */
	const int modBefore = (m < h ? 1 : 0);

	/* combinations between child, head and immediate context tags
	   Table 1 (c) McD, second part */

	/* collect information about start/end and context */
	int spt = -1; /* start next tag */
	int snt = -1; /* start previous tag */
	int ept = -1; /* end next tag */
	int ent = -1; /* end previous tag */
	int st = -1;  /* start tag */
	int et = -1;  /* end tag */
	if(start >= 0) {
	  st = x->get_token(start).coarse_tag();
	  if(start > 0)     { spt = x->get_token(start - 1).coarse_tag(); }
	  if(start < N - 1) { snt = x->get_token(start + 1).coarse_tag(); }
	}
	assert(end >= 0);
	et = x->get_token(end).coarse_tag();
	if(end > 0)     { ept = x->get_token(end - 1).coarse_tag(); }
	if(end < N - 1) { ent = x->get_token(end + 1).coarse_tag(); }

	/* CHANGED BY TERRY: Distinguish between the four possible
	   contexts: ++, --, +-, -+, and distinguish special cases:
	   root, adj.  Also, include backed-off versions: +?, ?+, -?,
	   ?- */

	if(start < 0) {
	  /* unary root dependency.  only generate contexts for the
	     non-root token */

	  /* end-1 + end + end+1 */
	  os.str("");
	  os << "depctxt-root[-+]:"
	     << ept << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());

	  /* backed-off : end-1 */
	  os.str("");
	  os << "depctxt-root[-?]:"
	     << ept << "+"
	     << et;
	  f.push_back(os.str());

	  /* backed-off : end+1 */
	  os.str("");
	  os << "depctxt-root[?+]:"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());
	} else if(end - start == 1) {
	  /* binary dependency with adjacent tokens.  only the outer
	     context (-+) is possible.  we use a special adjacent
	     dependency feature */

	  //    /* general -+ context feature */
	  //    os << "depctxt[-+]:" << modBefore << "+"
	  //       << spt << "+"
	  //       << st << "+"
	  //       << et << "+"
	  //       << ent;
	  //    addFeature(features, os.str(), mDepDict);

	  //    /* backed-off : general start-1 */
	  //    os.str("");
	  //    os << "depctxt[-?]:" << modBefore << "+"
	  //       << spt << "+"
	  //       << st << "+"
	  //       << et;
	  //    addFeature(features, os.str(), mDepDict);

	  //    /* backed-off : general end+1 */
	  //    os.str("");
	  //    os << "depctxt[?+]:" << modBefore << "+"
	  //       << st << "+"
	  //       << et << "+"
	  //       << ent;
	  //    addFeature(features, os.str(), mDepDict);

	  /* start-1 + start + end + end+1 */
	  os.str("");
	  os << "depctxt-adj[-+]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());

	  /* backed-off : start-1 */
	  os.str("");
	  os << "depctxt-adj[-?]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << et;
	  f.push_back(os.str());

	  /* backed-off : end+1 */
	  os.str("");
	  os << "depctxt-adj[?+]:" << modBefore << "+"
	     << st << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());
	} else if(end - start == 2) {
	  /* binary dependency with 1 token in between.  three
	     contexts (--, -+, ++) are possible */

	  /* start-1 + start + end-1 + end */
	  os.str("");
	  os << "depctxt[--]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << ept << "+"
	     << et;
	  f.push_back(os.str());

	  /* start-1 + start + end + end+1 */
	  os.str("");
	  os << "depctxt[-+]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());

	  /* start + start+1 + end + end+1 */
	  os.str("");
	  os << "depctxt[++]:" << modBefore << "+"
	     << st << "+"
	     << snt << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());

	  /* the backed-off features for this case and the general
	     case are the same, so we will do those afterward */
	} else {
	  /* binary dependency with >= 2 tokens in between.  all
	     contexts are possible */

	  /* start-1 + start + end-1 + end */
	  os.str("");
	  os << "depctxt[--]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << ept << "+"
	     << et;
	  f.push_back(os.str());

	  /* start-1 + start + end + end+1 */
	  os.str("");
	  os << "depctxt[-+]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());

	  /* start + start+1 + end-1 + end */
	  os.str("");
	  os << "depctxt[+-]:" << modBefore << "+"
	     << st << "+"
	     << snt << "+"
	     << ept << "+"
	     << et;
	  f.push_back(os.str());


	  /* start + start+1 + end + end+1 */
	  os.str("");
	  os << "depctxt[++]:" << modBefore << "+"
	     << st << "+"
	     << snt << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());

	  /* the backed-off features are handled below (since it is
	     the same case as the above) */
	}

	/* general 4-way backoff */
	if(end - start >= 2) {
	  /* backed-off : start-1 */
	  os.str("");
	  os << "depctxt[-?]:" << modBefore << "+"
	     << spt << "+"
	     << st << "+"
	     << et;
	  f.push_back(os.str());

	  /* backed-off : start+1 */
	  os.str("");
	  os << "depctxt[+?]:" << modBefore << "+"
	     << st << "+"
	     << snt << "+"
	     << et;
	  f.push_back(os.str());


	  /* backed-off : end-1 */
	  os.str("");
	  os << "depctxt[?-]:" << modBefore << "+"
	     << st << "+"
	     << ept << "+"
	     << et;
	  f.push_back(os.str());

	  /* backed-off : end+1 */
	  os.str("");
	  os << "depctxt[?+]:" << modBefore << "+"
	     << st << "+"
	     << et << "+"
	     << ent;
	  f.push_back(os.str());
	}
      }
    }
  }
}
