#include "FGToken.h"

#include "CharUtils.h"

using namespace std;

namespace egstra {
  void fg_token::phi(const sentence* x, list<string> F[],
		     const int word_limit,
		     const bool use_pos) {
    const int N = x->size();
    for(int i = 0; i < N; ++i) {
      list<string>& f_i = F[i];

      /* the token to be inspected */
      const token& tok = x->get_token(i);

      string wstr;
      string lstr;
      string cstr;
      string fstr;
      toString(tok.word(), wstr);
      toString(tok.lemma(), lstr);
      toString(tok.coarse_tag(), cstr);
      toString(tok.fine_tag(), fstr);

      const bool wvalid = (tok.word() < word_limit);

      /* the word */
      if(wvalid) {
	f_i.push_back("word:" + wstr);

	/* word + coarse POS */
	if(use_pos) {
	  f_i.push_back("word+ctag:" + wstr + "+" + cstr);
	}
      }

      /* the lemma, if available */
      if(tok.lemma() >= 0){
	f_i.push_back("lemma:" + lstr);
      }

      if(use_pos) {
	/* the coarse POS */
	f_i.push_back("ctag:" + cstr);

	/* the fine-grained POS, if available */
	if(tok.fine_tag() >= 0){
	  f_i.push_back("ftag:" + fstr);
	}
      }

      /* the morpho-syntactic features, if available */
      for(list<int>::const_iterator it = tok.morphos().begin();
	  it != tok.morphos().end(); it ++){
	string mstr;
	toString(*it, mstr);
	f_i.push_back("morpho:" + mstr);
	if(wvalid) {
	  f_i.push_back("word+morpho:" + wstr + "+" + mstr);
	}
      }
    }
  }
}
