#include "FGTokenContext.h"

#include "CharUtils.h"

#include <sstream>

using namespace std;

namespace egstra {
  void fg_token_context::phi(const sentence* x,
			     list<string> F[],
			     const int n,
			     const int word_limit,
			     const bool use_pos) {
    const int N = x->size();
    list<string>* Fnew = new list<string>[N];
    string* tstrs = (use_pos ? new string[N] : NULL);
    for(int i = 0; i < N; ++i) {
      /* create features based on replication of the existing
	 token features */
      list<string>& fnew = Fnew[i];
      for(int j = 1; j <= n; ++j) {
	string jstr;
	toString(j, jstr);
	/* previous context */
	if(i - j >= 0) {
	  string prefix = "p" + jstr + ":";
	  const list<string>& fprev = F[i - j];
	  list<string>::const_iterator it = fprev.begin();
	  const list<string>::const_iterator it_end = fprev.end();
	  for(; it != it_end; ++it) {
	    fnew.push_back(prefix + *it);
	  }
	}
	/* following context */
	if(i + j < N) {
	  string prefix = "n" + jstr + ":";
	  const list<string>& fnext = F[i + j];
	  list<string>::const_iterator it = fnext.begin();
	  const list<string>::const_iterator it_end = fnext.end();
	  for(; it != it_end; ++it) {
	    fnew.push_back(prefix + *it);
	  }
	}
      }

      if(use_pos) {
	/* collect tag string */
	const token& tok = x->get_token(i);
	toString(tok.coarse_tag(), tstrs[i]);
      }
    }

    for(int i = 0; i < N; ++i) {
      list<string>& fnew = Fnew[i];
      /* create features based on tag n-grams */
      for(int j = 1; j <= n; ++j) {
	/* previous n-gram */
	if(i - j >= 0) {
	  ostringstream os;
	  os << "ptags:";
	  for(int k = i - j; k <= i; ++k) {
	    os << "+" << tstrs[k];
	  }
	  fnew.push_back(os.str());
	}
	/* following n-gram */
	if(i + j < N) {
	  ostringstream os;
	  os << "ntags:";
	  for(int k = i + j; k >= i; --k) {
	    os << "+" << tstrs[k];
	  }
	  fnew.push_back(os.str());
	}
      }

      /* splice the new features into the current features.  note that
	 this is constant time for linked lists. */
      list<string>& f = F[i];
      f.splice(f.begin(), fnew);
    }

    delete [] Fnew;
    if(use_pos) { delete [] tstrs; }
  }
}
