#include "EGDuals.h"

#include "Example.h"
#include "Timer.h"
#include "OptUtils.h"

#include <iostream>
#include <stdlib.h>

#define myname "EGDuals"

using namespace std;

namespace egstra {
  egduals::egduals(const int N, const bool disk)
    : _theta(NULL), _mu(NULL), _logZ(NULL),
      _usedisk(disk), _N(N) {
    if(!_usedisk) {
      _theta = new const double*[_N];
      _mu = new const double*[_N];
      _logZ = new double[_N];
      for(int i = 0; i < _N; ++i) {
	_theta[i] = NULL; _mu[i] = NULL;
	_logZ[i] = 0.0;
      }
    } else {
      cerr << myname << " : disk-support not implemented" << endl;
      exit(1);
    }
  }

  egduals::~egduals() {
    if(_usedisk) {
      for(int i = 0; i < _N; ++i) {
	if(_theta[i] != NULL) { delete [] _theta[i]; }
	if(_mu[i] != NULL)    { delete [] _mu[i]; }
      }
      delete [] _theta;
      delete [] _mu;
      delete [] _logZ;
    }
  }


  void egduals::get(const int xid,
		    const double*& theta,
		    const double*& mu,
		    double& logZ) {
    if(!_usedisk) {
      assert(xid >= 0);
      assert(xid < _N);
      assert(_theta[xid] != NULL);
      assert(_mu[xid] != NULL);
      theta = _theta[xid];
      mu = _mu[xid];
      logZ = _logZ[xid];
    } else {
      cerr << myname << " : disk-support not implemented" << endl;
      exit(1);
    }
  }

  void egduals::set(const int xid,
		    const double* const theta,
		    const double* const mu,
		    const double logZ) {
    if(!_usedisk) {
      assert(xid >= 0);
      assert(xid < _N);
      assert(theta != NULL);
      assert(mu != NULL);
      _theta[xid] = theta;
      _mu[xid]    = mu;
      _logZ[xid]  = logZ;
    } else {
      cerr << myname << " : disk-support not implemented" << endl;
      exit(1);
    }
  }

  void egduals::discard(const double* theta,
			const double* mu) {
    /* if we are not using disk, there is nothing to do in this
       function */
    if(_usedisk) {
      delete [] theta;
      delete [] mu;
    }
  }
}

#undef myname
