/* EGDuals : a class that holds part-factored dual vectors.

   Author : Terry Koo
            maestro@csail.mit.edu */
#ifndef EGSTRA_EGDUALS_H
#define EGSTRA_EGDUALS_H

#include "Dataset.h"
#include "StructuredModel.h"
#include "FGen.h"
#include "Parameters.h"

namespace egstra {
  class egduals {
  private:
    const double** _theta;
    const double** _mu;
    double* _logZ;
    const bool _usedisk;
    const int _N;

  public:
    egduals(const int N, const bool disk = false);
    ~egduals();

    /* retrieve the duals for the given example.  this is currently
       done in memory but in the future, there may be an option to use
       the disk, so it is best to force usage of this function. */
    void get(const int xid,
	     const double*& theta,
	     const double*& mu,
	     double& logZ);
    /* store a set of duals for the given example. */
    void set(const int xid,
	     const double* const theta,
	     const double* const mu,
	     const double logZ);
    /* get rid of these dual vectors.  theta and mu must have been
       returned by a previous call to get() */
    void discard(const double* theta, const double* mu);

  };
}

#endif /* EGSTRA_EGDUALS_H */
