
/************************************************************
 *
 *   Author : Terry Koo
 *            maestro@csail.mit.edu
 *
 *   Class containing a number of boilerplate utilities for
 *   optimization algorithms
 *
 ************************************************************/
#ifndef EGSTRA_OPTUTILS_H
#define EGSTRA_OPTUTILS_H

#include "FGen.h"
#include "StructuredModel.h"
#include "Parameters.h"
#include "Dataset.h"

#include <vector>
#include <string>

namespace egstra {
  class optutils {
  public:
    /* create an array of 0/1 indicators indicating which parts are
       the "correct" parts for this example (i.e., those which match
       ex->y()).  NB: memory for this array should be deallocated with
       free(). */
    static char* get_correct(const example* const ex);

    /* count the number of parts in the predicted label yhat that
       match the gold-standard label for this example */
    static int count_matches(const example* const ex, const label& yhat);

    /* given the validation set, primals, and an output filename,
       classify the held-out data and write the labels to the file
       "max-labels.<t>".  also find min-risk labels using the
       marginals as in Goodman (1996) and write to "mbr-labels.<t>"
       the min-risk labelings can be disabled with minrisk=false. */
    static void classify_val_set(const parameters& V,
				 const dataset& data,
				 fgen& fg,
				 structured_model& M,
				 const std::string& dir,
				 const int t,
				 const bool minrisk = true);

    /* load the training and (optionally) test datasets into the given
       vectors.  if valfile == NULL then validation data is not
       loaded */
    static void load_datasets(structured_model& M,
			      const char* const trfile,
			      dataset& trdata,
			      const char* const valfile,
			      dataset& valdata);

    /* deallocate all examples and clear the dataset */
    static void clear_dataset(dataset& data);

    /* choose the i'th random example from the dataset, based on the
       randomization type */
    static const example* randselect(const int rand_type,
				     const int i,
				     dataset& data);
  };
}


#endif /* EGSTRA_OPTUTILS_H */
