/* DepFeats1 : feature generator for first-order dependency parsing

   Author : Mihai Surdeanu, Xavier Carreras, Terry Koo */

#ifndef EGSTRA_DEPFEATS1_H
#define EGSTRA_DEPFEATS1_H

#include "FGen.h"
#include "FeatureDictionary.h"
#include "Register.h"

namespace egstra {
  class depfeats1 : public fgen {
  private:
    /* whether the feature-dictionary is being generated */
    bool _generation_mode;
    /* flags for activation of various feature classes */
    bool _use_token;
    bool _use_token_context;
    bool _use_dependency;
    bool _use_dependency_context;
    bool _use_dependency_between;
    bool _use_dependency_distance;
    /* flags for classes of information */
    int _word_limit;
    bool _use_pos;
    /* number of dependency labels */
    int _K;
    /* mapping for token features */
    FeatureDictionary _tok_dict;
    /*  mapping for dependency features */
    FeatureDictionary _dep_dict;
    /* basic unlabeled-edge dimensionality */
    int _unlab_dim;

    void compute_dim();

  public:
    depfeats1() : fgen("DepFeats1"),
      _K(-1), _unlab_dim(-1) {}
    ~depfeats1();


    void usage(const char* const mesg = "");
    void process_options();

    void load_dictionaries();
    void save_dictionaries();

    /* return an array of feature vectors that correspond to the
       various parts.  specifically, phi(x)[r] should give the
       features associated with part r. */
    const struct fvec* phi(const pattern* const x);
    /* discard the features associated with the above */
    void discard(const struct fvec* const F,
		 const pattern* const x);
  };

  REGISTER_FGEN(dep1, depfeats1);
}

#endif /* EGSTRA_DEPFEATS1_H */
