/* FGen : abstract class representing a feature mapping

   Create a subclass with the virtual functions defined.

   Author : Terry Koo
            maestro@csail.mit.edu */

#ifndef EGSTRA_FGEN_H
#define EGSTRA_FGEN_H

#include <string>

#include "Pattern.h"
#include "FVec.h"

namespace egstra {
  class fgen {
  protected:
    const char* _name;
    int _dim;             /* feature dimensionality */
    std::string _dictdir; /* dictionary directory */

  public:
    fgen(const char* name)
      : _name(name), _dim(-1), _dictdir(0) {}
    virtual ~fgen() {}

    /* accessors */
    int dim() const          { return _dim; }
    const char* name() const { return _name; }

    /* handle the command line */
    virtual void usage(const char* const mesg = "") = 0;
    virtual void process_options() = 0;

    /* load and save feature mappings */
    virtual void load_dictionaries();
    virtual void save_dictionaries();

    /* return an array of feature vectors that correspond to the
       various parts.  specifically, phi(x)[r] should give the
       features associated with part r.  the return array of feature
       vectors will be deallocated using the delete[] operator, so it
       *must* be allocated using the c++-style new[] operator. */
    virtual const struct fvec* phi(const pattern* const x) = 0;

    /* return a single feature vector corresponding to the part at
       index r. */
    virtual const struct fvec* phi(const pattern* const x, const int r);

    /* these functions will be called when the features are done being
       used.  either of the two functions below will be called
       depending on which of the above functions was used to generate
       the feature vectors */
    virtual void discard(const struct fvec* const F,
			 const pattern* const x) = 0;
    virtual void discard(const struct fvec* const F,
			 const pattern* const x, const int r);
  };
}

#endif /* EGSTRA_FGEN_H */
