/* ProjDep1 : Edge-factorized projective dependency parser

   The argmax, partition, and marginals routines were written by
   Xavier Carreras.  The kbest routines were written by Terry Koo.

   Author : Xavier Carreras, Terry Koo */

#ifndef EGSTRA_PROJDEP1_H
#define EGSTRA_PROJDEP1_H

#include "StructuredModel.h"
#include "Register.h"

namespace egstra {
  class projdep1 : public structured_model {
  private:
    /* nested datatypes for inference routines */
    struct backp;
    class marg;
    /* walk the path of backpointers to recover the viterbi tree */
    void unravel_tree(label& y, const struct backp* BP,
		      const int h, const int m,
		      const bool complete, const int N);
    /* compute inside and outside scores for all parts */
    void inside(const double* const S, struct marg& M);
    void outside(const double* const S, struct marg& M);

  protected:
    int _K;          /* number of edge labels */
    bool _multiroot; /* whether parsing is multiroot or not */

  public:
    projdep1()
      : structured_model("PD1"),
      _K(-1), _multiroot(false) {}
    ~projdep1() {}

    void usage(const char* const mesg = "");
    void process_options();

    bool fread(FILE* in, pattern*& x, label*& y);
    void fwrite(FILE* out, const pattern* const x, const label& y);

    double argmax(const pattern* x, const double* const S, label& y);
    double partition(const pattern* x, const double* const S);
    double marginals(const pattern* x, const double* const S,
		     double* const M);
  };

  REGISTER_MODEL(projdep1, projdep1);
}

#endif /* EGSTRA_PROJDEP1_H */
