/************************************************************
 *
 *   Author : Terry Koo and Xavier Carreras
 *            {maestro,carreras}@csail.mit.edu
 *
 *   PercTrainer : trains a primal weight vector using the
 *                 structured (averaged) perceptron
 *
 ************************************************************/
#ifndef EGSTRA_PERCTRAINER_H
#define EGSTRA_PERCTRAINER_H

#include "Trainer.h"
#include "Parameters.h"
#include "Register.h"

namespace egstra {

  class perc_trainer : public trainer {
  private:

    /**** global algorithm parameters *****/
    /* primal parameters */
    parameters _W;
    /* sum over primal parameters */
    double* _Wsum;
    /* last-update timestamps for each parameter */
    int* _Wtime;

    /***** training invocation variables *****/
    /* style of randomization */
    int _rand_type;
    /* seed used for srandom() */
    int _seed;
    /* whether the primal parameters are averaged */
    bool _averaged;
    /* the required margin of separation */
    double _margin;
    /* number of examples processed on the current training run */
    int _num_processed;
    /* whether to write parameter vectors to disk */
    bool _writeparams;

    /* examine this example */
    void process_example(const example* const ex);

    /* flush all updates to the averaged parameters, bringing the
       averaged parameter vector completely up to date */
    void flush_avg();

  public:
    perc_trainer() : trainer() {}
    ~perc_trainer() {}

    void usage(const char* const mesg = "");
    void process_options();

    void train(dataset& trdata, dataset& valdata, const int T);

  };

  REGISTER_TRAINER(perc, perc_trainer);
}

#endif /* EGSTRA_PERCTRAINER_H */
