#include "Registry.h"

#include <iostream>

#include <assert.h>
#include <stdlib.h>

using namespace std;

namespace egstra {

  reg* registry::_registry = NULL;

  registry::registry(const char* type, const char* name, constructor f) {
    /* allocate the registry mapping here.  note that due to issues
       with the order of initialization/construction of static data
       members, it is not reliable to declare _registry as a normal
       object.  specifically, it is possible to arrive at this code
       without having initialized _registry (due to the fact that
       registry objects are instantiated at global scope; see
       Register.h).  therefore, we use a pointer and allocate it on
       demand. */
    if(_registry == NULL) { _registry = new reg(); }
    registry::add(type, name, f);
  }
  registry::~registry() {}


  void registry::add(const char* ctype, const char* cname, constructor f) {
    reg& R = *_registry;
    string type(ctype);
    string name(cname);
    const consmap::iterator end = R[type].end();
    consmap::iterator it = R[type].find(name);
    if(it != end) {
      cerr << "Registry : detected double-registration of "
	   << type << " \"" << name << "\"" << endl;
      exit(1);
    }
    R[type][name] = f;
  }

  void* registry::construct(const char* ctype, const char* cname) {
    reg& R = *_registry;
    string type(ctype);
    string name(cname);
    cerr << "Registry : constructing " << type
	 << " \"" << name << "\"" << endl;
    const consmap::iterator end = R[type].end();
    consmap::iterator it = R[type].find(name);
    if(it == end) {
      cerr << "Registry : no constructor for " << type
	   << " \"" << name << "\"" << endl;
      exit(1);
    }
    constructor f = R[type][name];
    return f();
  }

  void registry::getnames(const char* ctype, vector<string>& names) {
    reg& R = *_registry;
    string type(ctype);
    const reg::iterator end = R.end();
    reg::iterator it = R.find(type);
    if(it == end) {
      cerr << "Registry : invalid type \"" << type << "\"" << endl;
      exit(1);
    }
    consmap::iterator cit = R[type].begin();
    const consmap::iterator cend = R[type].end();
    for(; cit != cend; ++cit) {
      names.push_back(cit->first);
    }
  }
}
