/**
 * Contains all global constants for this project
 * @file Constants.h
 * @author Mihai Surdeanu
 */

#ifndef EGSTRA_CONSTANTS_H
#define EGSTRA_CONSTANTS_H

#define VERSION "0.1"

/* possible styles of randomization */
#define EGSTRA_RAND_UNIFORM          0
#define EGSTRA_RAND_PERMUTATION      1
#define EGSTRA_DETERMINISTIC         2

/* convert a rand-type to a string description */
#define RAND_LONG_GLOSS(randtype)				\
  (randtype == EGSTRA_RAND_UNIFORM				\
   ? "random with replacement"					\
   : (randtype == EGSTRA_RAND_PERMUTATION				\
      ? "random permutations"					\
      : (randtype == EGSTRA_DETERMINISTIC				\
	 ? "deterministic (cyclic)"				\
	 : "<unknown randomization, error!>")))			\

/* convert a rand-type to an abbreviation */
#define RAND_SHORT_GLOSS(randtype)				\
  (randtype == EGSTRA_RAND_UNIFORM ? "R"				\
   : (randtype == EGSTRA_RAND_PERMUTATION ? "P"			\
      : (randtype == EGSTRA_DETERMINISTIC ? "D"			\
	 : "<error>")))						\


/* multiplicative increment to per-example learning rates in the
   adaptive scheme */
#define EGSTRA_LRATE_DEFAULT_BOOST 1.05
/* fraction of examples to sample when automatically estimating the
   learning rate, and a minimum sample size as well */
#define EGSTRA_LRATE_SAMPLERATIO   0.1
/* proportion of cuts (relative to number of examples) permissible to
   consider a learning rate viable when estimating */
#define EGSTRA_LRATE_CUTRATIO      0.05
/* maximum number of learning-rate halvings per example */
#define EGSTRA_LRATE_MAXCUTS       5
/* convergence tolerance for the relative duality gap (= gap/dual) */
#define EGSTRA_CONV_TOL  1e-12

/* default beta value */
#define EGSTRA_BETA_DEFAULT 3

// Maximum line size in a CoNLL-X file
#define MAX_CONLLX_LINE 32 * 1024

// Maximum line size in a feature dictionary file
#define MAX_FEATDICT_LINE 32 * 1024

// Discard all features with frequency below this threshold
#define UNKNOWN_FEATURE_THRESHOLD 0

// Threshold for the token distance within a deependency
#define TOKEN_DISTANCE_THRESHOLD 41
// Threshold for the number of verbs between the deependency tokens
#define VERB_COUNT_THRESHOLD 4
// Threshold for the number of commas between the deependency tokens
#define PUNC_COUNT_THRESHOLD 4
// Threshold for the number of coordinations between the deependency tokens
#define CC_COUNT_THRESHOLD 4

#endif /* EGSTRA_CONSTANTS_H */
